#include "stdafx.h"
#include "Menu.h"
#include "Config.h"

HMENU hMainMenu;
MENUTAG* firstTag;


void loadMenuResource(MSXML2::IXMLDOMNodePtr curNode, MENUTAG* menuTag, bool isTopMenu) {
	_bstr_t bstr;
	MSXML2::IXMLDOMNodePtr myNode;
	LPWSTR iconPath;
	int iconIndex;


	bstr = curNode->text;
	menuTag->command = (LPWSTR)malloc(sizeof(LPWSTR) * lstrlen(bstr));
	lstrcpy(menuTag->command, bstr);
	wcstok_s(menuTag->command, L"|", &(menuTag->arguments));
	//SysFreeString(bstr);

	bstr = curNode->attributes->getNamedItem(L"text")->text;
	menuTag->text = (LPWSTR)malloc(sizeof(LPWSTR) * lstrlen(bstr));
	lstrcpy(menuTag->text, bstr);
	//SysFreeString(bstr);

	myNode = curNode->attributes->getNamedItem(L"index");
	if (myNode == NULL) {
		iconIndex = 0;
	}
	else {
		bstr = myNode->text;
		iconIndex = _wtoi((LPWSTR)bstr);
		//SysFreeString(bstr);
	}

	myNode = curNode->attributes->getNamedItem(L"icon");
	if (myNode == NULL) {
		iconPath = menuTag->command;
	}
	else {
		bstr = myNode ->text;
		iconPath = (LPWSTR)malloc(sizeof(LPWSTR) * lstrlen(bstr));
		lstrcpy(iconPath, bstr);
		//SysFreeString(bstr);
	}

	myNode = curNode->attributes->getNamedItem(L"size");
	if (myNode == NULL) {
		menuTag->iconSize = Config_GetDefIconSize(isTopMenu);
	}
	else {
		bstr = myNode->text;
		menuTag->iconSize = _wtoi((LPWSTR)bstr);
		//SysFreeString(bstr);
	}

	if (menuTag->iconSize > 20) {
		if (ExtractIconEx(iconPath, iconIndex, &(menuTag->icon), NULL, 1) == 0) {
			ExtractIconEx(iconPath, iconIndex, NULL, &(menuTag->icon), 1);
		}
	}
	else {
		if (ExtractIconEx(iconPath, iconIndex, NULL, &(menuTag->icon), 1) == 0) {
			ExtractIconEx(iconPath, iconIndex, &(menuTag->icon), NULL, 1);
		}
	}

	//menuTag->icon = ExtractIcon(GetModuleHandle(NULL), iconPath, iconIndex);
	if (iconPath != menuTag->command)
		free(iconPath);
	

	menuTag->isTopMenu = isTopMenu;
	menuTag->nextTag = nullptr;
}

void loadMenuTree(MSXML2::IXMLDOMNodePtr curNode, HMENU menu, UINT* menuIndex, bool isTopMenu, MENUTAG* prevTag) {
	MENUTAG* menuTag;

	if (strcmp(curNode->nodeName, "item") == 0) {
		menuTag = (MENUTAG*)malloc(sizeof(MENUTAG));
		loadMenuResource(curNode, menuTag, isTopMenu);
		AppendMenu(menu, MF_OWNERDRAW, *menuIndex, (LPCWSTR)menuTag);
		
		MENUITEMINFO  info;
		info.cbSize = sizeof(MENUITEMINFO);
		info.fMask = MIIM_DATA;
		GetMenuItemInfo(menu, *menuIndex, false, &info);
		MENUTAG* t = (MENUTAG*)info.dwItemData;


		if (prevTag == nullptr) {
			firstTag = menuTag;
		}
		else {
			prevTag->nextTag = menuTag;
		}
		*menuIndex = *menuIndex + 1;
	}

		
	if (strcmp(curNode->nodeName, "separator") == 0) {
		AppendMenu(menu, MF_OWNERDRAW | MF_SEPARATOR, isTopMenu ? SPLITTER_TOPLEVEL : SPLITTER_NORMAL, nullptr);
	}

	if (strcmp(curNode->nodeName, "submenu") == 0) {
		HMENU subMenu = CreatePopupMenu();
		menuTag = (MENUTAG*)malloc(sizeof(MENUTAG));
		loadMenuResource(curNode, menuTag, isTopMenu);
		AppendMenu(menu, MF_OWNERDRAW | MF_POPUP, (UINT_PTR)subMenu, (LPCWSTR)menuTag);
		if (prevTag == nullptr) {
			firstTag = menuTag;
		}
		else {
			prevTag->nextTag = menuTag;
		}


		for (int i = 0; i != curNode->childNodes->length; ++i) {
			loadMenuTree(curNode->childNodes->item[i], subMenu, menuIndex, false, menuTag);
		}
	}

}



void Menu_Build() {
	UINT menuIndex;

	MSXML2::IXMLDOMNodeListPtr spNodeList = Config_Root()->childNodes;
	menuIndex = 1;
	for (int i = 0; i != spNodeList->length; ++i) {
		MSXML2::IXMLDOMNodePtr curNode = spNodeList->item[i];
		
		loadMenuTree(curNode, hMainMenu, &menuIndex, true, firstTag);
	}
}


void Menu_Init() {
	Menu_Destroy();

	hMainMenu = CreatePopupMenu();

	MENUINFO menuInfo;
	memset(&menuInfo, 0, sizeof(MENUINFO));
	menuInfo.cbSize = sizeof(MENUINFO);
	menuInfo.fMask = MIM_STYLE;
	menuInfo.dwStyle = MNS_NOTIFYBYPOS;
	SetMenuInfo(hMainMenu, &menuInfo);

	Menu_Build();

	/*
	MENUINFO menuInfo;
	memset(&menuInfo, 0, sizeof(MENUINFO));
	menuInfo.cbSize = sizeof(MENUINFO);
	menuInfo.fMask = MIM_BACKGROUND;
	HBITMAP h = (HBITMAP)LoadImage(GetModuleHandle(NULL), L"D:\\1.bmp", IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE);
	//HBITMAP bmp = LoadBitmap(hInst,MAKEINTRESOURCE(h));
	//UINT err = GetLastError();
	menuInfo.hbrBack = CreatePatternBrush(h);
	SetMenuInfo(hMenu, &menuInfo); 
	*/
}



void Menu_Show(HWND hWnd, WORD xPos, WORD yPos) {
	CalcScale();

	SetForegroundWindow(hWnd);

	RECT myRect;
	GetWindowRect(hWnd, &myRect);
	SystemParametersInfo(SPI_GETWORKAREA, 0, &myRect, 0);
	TrackPopupMenuEx(hMainMenu, TPM_RIGHTBUTTON | TPM_VERNEGANIMATION | TPM_NONOTIFY, 0, myRect.bottom, hWnd , NULL);
	return;
}

void Menu_Action(HMENU hMenu, WORD MenuPos) {
	MENUITEMINFO  info;
	info.cbSize = sizeof(MENUITEMINFO);
	info.fMask = MIIM_DATA;
	GetMenuItemInfo(hMenu, MenuPos, true, &info);
	MENUTAG* t = (MENUTAG*)info.dwItemData;

	ShellExecute(0, L"open", t->command, t->arguments, L"", SW_SHOWNORMAL);

}

void Menu_Destroy() {
	if (hMainMenu != nullptr) {
		for (MENUTAG* curNode = firstTag; curNode != nullptr; ) {
			free(curNode->command);
			free(curNode->text);
			DestroyIcon(curNode->icon);
			MENUTAG* f = curNode;
			curNode = curNode->nextTag;
			free(f);
		}
		firstTag = nullptr;
		DestroyMenu(hMainMenu);
	}
}
